<?php
/**
 * API: AIKPT - SYNC POST FROM AIKPT TO NUKEVIET
 * Description:	Synchronize articles between AIKPT and NUKEVIET platform
 * Version:	1.0.0
 * Author: John Luke - aiktp.com 
 * Author URI: https://aiktp.com
 * Text Domain: aiktp
 * Plugin URI: http://aiktp.com
 */

namespace NukeViet\Api;

ini_set('display_errors', 1);
ini_set('display_startup_errors', 1);
error_reporting(E_ALL);

if (!defined('NV_ADMIN') or !defined('NV_MAINFILE')) {
    exit('Stop!!!');
}

class AIKTP implements IApi
{
    private $result;

    /**
     * getAdminLev()
     *
     * @return int
     */
    public static function getAdminLev()
    {
        return Api::ADMIN_LEV_MOD;
    }

    /**
     * getCat()
     *
     * @return string
     */
    public static function getCat()
    {
        return 'webtools';
    }

    /**
     * setResultHander()
     */
    public function setResultHander(ApiResult $result)
    {
        $this->result = $result;
    }

    /**
     * execute()
     *
     * @return mixed
     */
    public function execute()
    {
		global $db;
		$method = $_POST['method'];
		if ($method == "updateNews"){
			$postData = $db->query("SELECT * FROM `" . NV_PREFIXLANG . "_news_rows` WHERE id= '".$_POST['postId']."'")->fetch();
			if (empty($postData['id'])) $method = 'insertNews';
		}
		
        switch ($method){
		   case 'getCategories': return $this->getCategories(); break;
		   case 'insertNews': 
				$inputData['title'] =  $_POST['title'];
				$inputData['imgURL'] =  $_POST['imgURL'];
				$inputData['content'] =  $_POST['content'];
				$inputData['keywords'] =  $_POST['tags'];
				return $this->insertNews($inputData);
			break;
			 case 'updateNews': 
				$inputData['postId'] =  $_POST['postId'];
				$inputData['title'] =  $_POST['title'];
				$inputData['imgURL'] =  $_POST['imgURL'];
				$inputData['content'] =  $_POST['content'];
				$inputData['catid'] =  $_POST['catid'];
				$inputData['keywords'] =  $_POST['tags'];
				return $this->updateNews($inputData);
			break;
		   case 'downloadRemoteImage': 
				$imgURL = $_POST['imgURL'];
				$title = $_POST['title'];
				return $this->downloadRemoteImage($imgURL,$title);
			break;
			case 'getNewsLink': 
				$postId = $_POST['postId'];
				return $this->getNewsPermaLink($postId);
			break;
			case 'getPostByURL': 
				$url = $_POST['url'];
				return $this->getPostByURL($url);
			break;
			case 'getPostByKeywords': 
				$keyword = $_POST['keyword'];
				return $this->getPostByKeywords($keyword);
			break;
	   }
	   
    }
	private function getPostByKeywords($keyword){
	    global $db,$global_config;
		$last = explode(".",$last);
		$postId = $last[0];
		
		$_keyword = strtolower($keyword);
		$_keyword = str_replace("'","",$_keyword);
		

		$postDatas = $db->query("SELECT post.*, cat.alias as cat_alias FROM `" . NV_PREFIXLANG . "_news_rows`  as post
								LEFT JOIN `" . NV_PREFIXLANG . "_news_detail` ON `post`.id=`" . NV_PREFIXLANG . "_news_detail`.id 
								LEFT JOIN `" . NV_PREFIXLANG . "_news_cat` as cat ON `cat`.catid = `post`.catid
								WHERE `" . NV_PREFIXLANG . "_news_detail`.keywords LIKE '%".$_keyword."%'")->fetchAll();
		$postData = array();
		foreach ($postDatas as $post){
			$item['postTitle'] = $post['title'];
			$item['catid'] = $post['catid'];
			$item['postId'] = $post['id'];
			
			if ($global_config['rewrite_enable']){
				   $permalink = $global_config['site_url'].'/vi/news/'.$post['cat_alias'].'/'.$post['alias'].'-'.$post['id'].$global_config['rewrite_exturl'];
			   }else{
				   $permalink =  $global_config['site_url'].'/index.php?language=vi&nv=news&op='.$post['cat_alias'].'/'.$post['alias'].'-'.$post['id'].$global_config['rewrite_exturl'];
			   }
			   $item['link'] = $permalink;
			   
			$postData[] = $item;
		}
		
		
		$output['status'] = 'success';
		$output['posts'] = $postData;
		return json_encode($output);
	}
	private function getPostByURL($url){
	    global $db,$global_config;
		
		$split = explode("-",$url);
		$last = end($split);
		if ($last){
			$last = explode(".",$last);
			$postId = $last[0];
			
			
			$postData = $db->query("SELECT * FROM `" . NV_PREFIXLANG . "_news_rows` WHERE id= '".$postId."'")->fetch();
			$postDetail = $db->query("SELECT * FROM `" . NV_PREFIXLANG . "_news_detail` WHERE id= '".$postId."'")->fetch();
			$outData['title']  = $postData['title'];
			$outData['content']  = $postDetail['bodyhtml'];
			$outData['postId']  = $postId;
			if (!empty($postData['homeimgfile'])) $outData['thumbnail']  = $global_config['site_url']."/uploads/".$postData['homeimgfile'];
			$output['data'] = $outData;
			$output['status'] = 'success';
			$output['status'] = 'success';
			return json_encode($output);
		}else{
			$output['status'] = 'error';
		}
		return $output;
	}
	private function getCategories(){
	    global $db;
		$rows = $db->query('SELECT catid,parentid,title,alias FROM ' . NV_PREFIXLANG . '_news_cat')->fetchAll();
		
		$output['status'] = 'success';
		$output['cats'] = $rows;
        return json_encode($output);
	}

   private function getNewsPermaLink($postId){
	   global $db,$global_config;
	   
	   $postData = $db->query("SELECT * FROM `" . NV_PREFIXLANG . "_news_rows` WHERE id= '".$postId."'")->fetch();
	   $catData = $db->query("SELECT * FROM `" . NV_PREFIXLANG . "_news_cat` WHERE catid= '".$postData['catid']."'")->fetch();
	   if ($global_config['rewrite_enable']){
		   $permalink = $global_config['site_url'].'/vi/news/'.$catData['alias'].'/'.$postData['alias'].'-'.$postData['id'].$global_config['rewrite_exturl'];
	   }else{
		   $permalink =  $global_config['site_url'].'/index.php?language=vi&nv=news&op='.$catData['alias'].'/'.$postData['alias'].'-'.$postData['id'].$global_config['rewrite_exturl'];
	   }
	   $output['permalink'] = $permalink;
	   return json_encode($output);
   }
   private function updateNews($input){
	   global $db,$nv_Cache;
		$postData = $db->query("SELECT * FROM `" . NV_PREFIXLANG . "_news_rows` WHERE id= '".$input['postId']."'")->fetch();
		$title = trim( strip_tags( $input['title']) );

		$textContent =  strip_tags($input['content']);
		$textArr = explode(" ",$textContent);
		if (count($textArr) > 40) {
			$desArr = array_slice($textArr,0, 40);
			$description = implode(" ",$desArr);
		}else{
			$description = $textContent;
		}
	  
		$homeimgfile = '';
		if (!empty($input['imgURL'])){
			$imgData = $this->downloadRemoteImage($input['imgURL'],$title,'',true);
			$imgData = json_decode($imgData,true);
			$homeimgfile = $imgData['fileURL'];
			$thumbnailUrl = $imgData['baseURL'];
			
			$sourceFilename = NV_UPLOADS_REAL_DIR.'/news/'.$homeimgfile;
			$alias = change_alias( $title );
			$toTmpFileName = NV_UPLOADS_REAL_DIR.'/../data/tmp/news_'.$input['postId'].'_'.$alias.'_500-0.jpg';
			$copied = copy($sourceFilename,$toTmpFileName);
			$nv_Cache->delMod('news');
		}
		
		if (!empty($input['catid'])) {
			$array['catid'] = $input['catid'];
			$array['listcatid'] = $input['catid'];
		}
		if (!empty($homeimgfile)) $update['homeimgfile'] = $homeimgfile;
		
		$update['title'] = $title;
		$update['hometext'] = $description;
		$update['edittime'] = NV_CURRENTTIME;
		$newsRowsSQL = "";
		foreach ($update as $k=>$v){
			$newsRowsSQL .= "`".$k."` = '".$v."', ";
		}
		
		$newsRowsSQL = substr($newsRowsSQL, 0, -2);
		
		$sql_newsRows = "UPDATE `" . NV_PREFIXLANG . "_news_rows` SET ".$newsRowsSQL." WHERE id=".$input['postId'];
		$db->exec( $sql_newsRows );
		
		
		$sql_news_1 = "UPDATE `" . NV_PREFIXLANG . "_news_1` SET ".$newsRowsSQL." WHERE id=".$input['postId'];
		$db->exec( $sql_news_1 );
		
		
		$updateDetail = array();
		if (!empty($input['content'])) {
			
			$imglist = $this->extractImages($input['content']);
			$imgData_01 = array();
			if (in_array($input['imgURL'],$imglist)){
				if (($key = array_search($input['imgURL'], $imglist)) !== false) {
					unset($imglist[$key]);
					$imgData_01[] = ['url'=> $input['imgURL'], 'baseURL' => $imgData['baseURL']];
				}
			}
			$imgData = array();
			$imgData = $this->multiDownloadImages($imglist,$title);
			if (!empty($imgData_01)) $imgData = array_merge($imgData,$imgData_01);
			
			foreach ($imgData as $img){
				if (!empty($img['url']) && !empty($img['baseURL']))
					$input['content'] = str_replace($img['url'],$img['baseURL'],$input['content']);
			}
			$updateDetail['bodyhtml'] = $input['content'];
		}
		if (!empty($input['keywords'])) $updateDetail['keywords'] = $input['keywords'];
		
		if (!empty($updateDetail)){
			$newsRowsSQL_DETAIL = "";
			foreach ($updateDetail as $k=>$v){
				$newsRowsSQL_DETAIL .= "`".$k."` = '".$v."', ";
			}
			$newsRowsSQL_DETAIL = substr($newsRowsSQL_DETAIL, 0, -2);
			$sql_news_detail = "UPDATE `" . NV_PREFIXLANG . "_news_detail` SET ".$newsRowsSQL_DETAIL." WHERE id= ".$input['postId'];
			$db->exec( $sql_news_detail );
		}
		
		
		$output['status'] = "success";
		$link = json_decode($this->getNewsPermaLink($input['postId']),true);
		$output['permalink'] = $link['permalink'];
		if (!empty( $thumbnailUrl)) $output['thumbnail'] = $thumbnailUrl; 
		$output['postId'] = $input['postId'];
		
		return json_encode($output);
		
   }
   private function extractImages($html){
		$html = stripslashes($html);
		preg_match_all('/<img[^>]+src\s*=\s*["\']([^"\']+)["\']/i', $html, $matches);
		return $matches[1]; 
  }
   private function insertNews($input){
		global $db,$nv_Cache;
		$title = trim( strip_tags( $input['title']) );
		$alias = change_alias( $title );
		
		$textContent =  strip_tags($input['content']);
		$textArr = explode(" ",$textContent);
		if (count($textArr) > 40) {
			$desArr = array_slice($textArr,0, 40);
			$description = implode(" ",$desArr);
		}else{
			$description = $textContent;
		}

	
		 $array['title'] = $title;
		 $array['alias'] = $alias;
		 $array['hometext'] = $description;
		 $array['catid'] = $input['catid']?$input['catid']:1;
		 $array['listcatid'] = $input['catid']?$input['catid']:'1,5';
		 $array['topicid'] = 0;
		 $array['author'] = '';
		 $array['sourceid'] = 0;
		 $array['addtime'] = NV_CURRENTTIME;
		 $array['edittime'] = NV_CURRENTTIME;
		 $array['status'] = 1;
		 $array['exptime'] = 0;
		 $array['archive'] = 2;
		
		 $homeimgfile = '';
		 if (!empty($input['imgURL'])){
			$imgData = $this->downloadRemoteImage($input['imgURL'],$title,'',true);
			$imgData = json_decode($imgData,true);
			$homeimgfile = $imgData['fileURL'];
			//copy file to tmp 
			
			$thumbnailUrl = $imgData['baseURL'];
		 }
		 
		
		
		$sql = "INSERT INTO `" . NV_PREFIXLANG . "_news_rows` 
		(`id`, `catid`, `listcatid`, `topicid`, `admin_id`, `author`, `sourceid`, `addtime`, `edittime`, `status`, `publtime`, `exptime`, `archive`, `title`, `alias`,`hometext`,`homeimgfile`,`homeimgthumb`, `inhome`, `allowed_comm`, `allowed_rating`, `hitstotal`, `hitscm`, `total_rating`, `click_rating`) VALUES (
			NULL, 
			'" . intval($array['catid']) . "',
			'" . mysql_escape_string($array['listcatid']) . "',
			'" . intval($array['topicid']) . "',
			'1',
			'" . mysql_escape_string($array['author']) . "',
			'" . intval($array['sourceid']) . "',
			'" . intval($array['addtime']) . "',
			'" . intval($array['edittime']) . "',
			'" . intval($array['status']) . "',
			'" . intval($array['addtime']) . "',
			'" . intval($array['exptime']) . "', 
			'" . intval($array['archive']) . "',
			'" . mysql_escape_string($array['title']) . "',
			'" . mysql_escape_string($array['alias']) . "',
			'" . mysql_escape_string($array['hometext']) . "',
			'" .$homeimgfile. "',
			'2',
			'1',  
			'" . intval(1) . "', 
			'" . intval(1) . "', 
			'" . intval(1) . "',  
			'" . intval(1) . "',  
			'" . intval(0) . "',  
			'" . intval(0) . "'
		)";
		
		$db->exec( $sql );
		$postId = $output['postId'] =  $db->lastInsertId();
		
		$sql = "INSERT INTO `" . NV_PREFIXLANG . "_news_1` 
		(`id`, `catid`, `listcatid`, `topicid`, `admin_id`, `author`, `sourceid`, `addtime`, `edittime`, `status`, `publtime`, `exptime`, `archive`, `title`, `alias`,`hometext`,`homeimgfile`,`homeimgthumb`, `inhome`, `allowed_comm`, `allowed_rating`, `hitstotal`, `hitscm`, `total_rating`, `click_rating`) VALUES (
			'".$postId."', 
			'" . intval($array['catid']) . "',
			'" . mysql_escape_string($array['listcatid']) . "',
			'" . intval($array['topicid']) . "',
			'1',
			'" . mysql_escape_string($array['author']) . "',
			'" . intval($array['sourceid']) . "',
			'" . intval($array['addtime']) . "',
			'" . intval($array['edittime']) . "',
			'" . intval($array['status']) . "',
			'" . intval($array['addtime']) . "',
			'" . intval($array['exptime']) . "', 
			'" . intval($array['archive']) . "',
			'" . mysql_escape_string($array['title']) . "',
			'" . mysql_escape_string($array['alias']) . "',
			'" . mysql_escape_string($array['hometext']) . "',
			'" .$homeimgfile. "',
			'2',
			'1',  
			'" . intval(1) . "', 
			'" . intval(1) . "', 
			'" . intval(1) . "',  
			'" . intval(1) . "',  
			'" . intval(0) . "',  
			'" . intval(0) . "'
		)";
		$db->exec( $sql );
		
		$imglist = $this->extractImages($input['content']);
		$imgData_01 = array();
		if (in_array($input['imgURL'],$imglist)){
			if (($key = array_search($input['imgURL'], $imglist)) !== false) {
				unset($imglist[$key]);
				$imgData_01[] = ['url'=> $input['imgURL'], 'baseURL' => $imgData['baseURL']];
			}
		}
		$imgData = array();
		$imgData = $this->multiDownloadImages($imglist,$title);
		if (!empty($imgData_01)) $imgData = array_merge($imgData,$imgData_01);
		foreach ($imgData as $img){
			if (!empty($img['url']) && !empty($img['baseURL'])){
				$input['content'] = str_replace($img['url'],$img['baseURL'],$input['content']);
			}
		}		
		
		$sqlNewsDetail = "INSERT INTO `" . NV_PREFIXLANG . "_news_detail`  (`id`,`description`,`bodyhtml`,`keywords`,`imgposition`) VALUES (
		'".$output['postId']."',
		'',
		'".mysql_escape_string($input['content'])."',
		'".mysql_escape_string($input['keywords'])."',
		'0'
		)";
		
		$db->exec( $sqlNewsDetail );
		
		
		$sourceFilename = NV_UPLOADS_REAL_DIR.'/news/'.$homeimgfile;
		$toTmpFileName = NV_UPLOADS_REAL_DIR.'/../data/tmp/news_'.$postId.'_'.$alias.'_500-0.jpg';
		$copied = copy($sourceFilename,$toTmpFileName);

		$nv_Cache->delMod('news');
		$output['status'] = "success";
		$link = json_decode($this->getNewsPermaLink($postId),true);
		$output['permalink'] = $link['permalink'];
		if (!empty( $thumbnailUrl)) $output['thumbnail'] = $thumbnailUrl; 
		$output['postId'] = $postId;
		return json_encode($output);
		
	}
	private function downloadRemoteImage($imgURL,$title='',$postId='',$copyToTmp = false){
		
		global $global_config;
		$site_url = $global_config['site_url'];
		$imgPath = parse_url($imgURL)['path'];
		$extension = end(explode('.', $imgPath));
		
		if (empty($extension)) $extension = "jpg";
		$basename = basename( $imgPath );		
		$mod_name = 'news';
		$folder = 'aiktp';
		if ( ! file_exists( NV_UPLOADS_REAL_DIR . '/news/aiktp' ) )
		{
			nv_mkdir( NV_UPLOADS_REAL_DIR . '/news', $folder );
		}
		if (empty($title)){
			$name = $basename;
		}else{
			$name = strtolower(change_alias($title)).".".$extension;
		}
		$thumb_name = "";
		$block_name = "";
		$baseURL = "";
		if (file_exists( NV_UPLOADS_REAL_DIR . '/news/aiktp/' . $name )){
			$baseURL =  $site_url. '/uploads/news/aiktp/' . $name ;
			
		}else{
			$img_filename = NV_UPLOADS_REAL_DIR . '/news/aiktp/' . $name;
			$ch = curl_init($imgURL);
			$fp = fopen($img_filename, 'wb');
			try{
				curl_setopt($ch, CURLOPT_FILE, $fp);
				curl_setopt($ch, CURLOPT_HEADER, 0);
				curl_setopt($ch, CURLOPT_TIMEOUT, 30);
				curl_setopt($ch, CURLOPT_FOLLOWLOCATION, 1);
				curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);
				curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, 0);
				curl_exec($ch);
				$rescode = curl_getinfo($ch, CURLINFO_HTTP_CODE); 
				curl_close($ch);
				
				
				if ($rescode == 200  && filesize($img_filename) > 100){
					$baseURL =  $site_url. '/uploads/news/aiktp/' . $name ;
					
				}
			}catch (Exception $e) {
				//echo $e;
			}
		}
		$output['status'] = 'success';
		$output['postId'] = $postId;
		$output['imgURL'] = $imgURL;
		$output['baseURL'] = $baseURL;
		$output['fileURL'] = 'aiktp/'.$name;
		return json_encode($output);
	}
	function multiDownloadImages($input,$title) {
		global $global_config;
		$output = [];
		$mh = curl_multi_init();
		$handles = [];

		foreach ($input as $index => $url) {
			if (strpos($url,$global_config['site_url']) === false){
			
				$imgPath = parse_url($url)['path'];
				$extension = end(explode('.', $imgPath));
				if (empty($extension)) $extension = "jpg";
				
				$filename = ($index + 1)."_".strtolower(change_alias($title)).".".$extension;
				$img_file_real_name = NV_UPLOADS_REAL_DIR . '/news/aiktp/' . $filename;
				
				$baseURL = $global_config['site_url']. '/uploads/news/aiktp/' . $filename;
				
				$ch = curl_init($url);
				$fp = fopen($img_file_real_name, 'w');
				curl_setopt($ch, CURLOPT_FILE, $fp);
				curl_setopt($ch, CURLOPT_HEADER, 0);
				curl_multi_add_handle($mh, $ch);
				$handles[(int)$ch] = ['ch' => $ch,'url' => $url,'img_file_real_name' => $img_file_real_name, 'baseURL' => $baseURL, 'fp' => $fp];
			}
		}

		$active = null;

		// Execute the multi handle
		do {
			$mrc = curl_multi_exec($mh, $active);
		} while ($mrc == CURLM_CALL_MULTI_PERFORM);

		while ($active && $mrc == CURLM_OK) {
			if (curl_multi_select($mh) != -1) {
				do {
					$mrc = curl_multi_exec($mh, $active);
				} while ($mrc == CURLM_CALL_MULTI_PERFORM);
			}
		}

		// Close the handles and clean up
		foreach ($handles as $handle) {
			curl_multi_remove_handle($mh, $handle['ch']);
			fclose($handle['fp']);
			curl_close($handle['ch']);
			$output[] = ['url' => $handle['url'], 'baseURL' => $handle['baseURL']];
		}

		curl_multi_close($mh);

		return $output;
	}
    
}
